@@ -24,16 +24,18 @@
 -- catalog tables
 --
 
+
 CREATE TABLE ag_graph (
+  graphid oid NOT NULL,
   name name NOT NULL,
   namespace regnamespace NOT NULL
-) WITH (OIDS);
+);
+
+CREATE UNIQUE INDEX ag_graph_graphid_index ON ag_graph USING btree (graphid);
 
 -- include content of the ag_graph table into the pg_dump output
 SELECT pg_catalog.pg_extension_config_dump('ag_graph', '');
 
-CREATE UNIQUE INDEX ag_graph_oid_index ON ag_graph USING btree (oid);
-
 CREATE UNIQUE INDEX ag_graph_name_index ON ag_graph USING btree (name);
 
 CREATE UNIQUE INDEX ag_graph_namespace_index
@@ -46,24 +48,26 @@ CREATE DOMAIN label_id AS int NOT NULL CHECK (VALUE > 0 AND VALUE <= 65535);
 CREATE DOMAIN label_kind AS "char" NOT NULL CHECK (VALUE = 'v' OR VALUE = 'e');
 
 CREATE TABLE ag_label (
+
   name name NOT NULL,
   graph oid NOT NULL,
   id label_id,
   kind label_kind,
   relation regclass NOT NULL,
-  seq_name name NOT NULL
-) WITH (OIDS);
+  seq_name name NOT NULL,
+  CONSTRAINT fk_graph_oid
+    FOREIGN KEY(graph)
+    REFERENCES ag_graph(graphid)
+);
 
 -- include content of the ag_label table into the pg_dump output
 SELECT pg_catalog.pg_extension_config_dump('ag_label', '');
 
-CREATE UNIQUE INDEX ag_label_oid_index ON ag_label USING btree (oid);
-
 CREATE UNIQUE INDEX ag_label_name_graph_index
 ON ag_label
 USING btree (name, graph);
 
-CREATE UNIQUE INDEX ag_label_graph_id_index
+CREATE UNIQUE INDEX ag_label_graph_oid_index
 ON ag_label
 USING btree (graph, id);
 
@@ -3145,6 +3149,14 @@ CALLED ON NULL INPUT
 PARALLEL SAFE
 AS 'MODULE_PATHNAME', 'agtype_build_map_noargs';
 
+CREATE FUNCTION ag_catalog.agtype_build_map_nonull(VARIADIC "any")
+RETURNS agtype
+LANGUAGE c
+STABLE
+CALLED ON NULL INPUT
+PARALLEL SAFE
+AS 'MODULE_PATHNAME';
+
 --
 -- There are times when the optimizer might eliminate
 -- functions we need. Wrap the function with this to
@@ -3543,6 +3555,14 @@ STABLE
 PARALLEL SAFE
 AS 'MODULE_PATHNAME';
 
+CREATE FUNCTION ag_catalog.age_isempty(agtype)
+RETURNS boolean
+LANGUAGE c
+IMMUTABLE
+RETURNS NULL ON NULL INPUT
+PARALLEL SAFE
+AS 'MODULE_PATHNAME';
+
 CREATE FUNCTION ag_catalog.age_label(agtype)
 RETURNS agtype
 LANGUAGE c
@@ -4146,10 +4166,10 @@ IMMUTABLE
 PARALLEL SAFE
 AS 'MODULE_PATHNAME';
 
-CREATE FUNCTION ag_catalog.age_unnest(agtype, block_types boolean = false)
-    RETURNS SETOF agtype
-    LANGUAGE c
-    IMMUTABLE
+CREATE FUNCTION ag_catalog.age_unnest(agtype)
+RETURNS SETOF agtype
+LANGUAGE c
+IMMUTABLE
 PARALLEL SAFE
 AS 'MODULE_PATHNAME';
 
@@ -4174,6 +4194,19 @@ CALLED ON NULL INPUT
 PARALLEL SAFE
 AS 'MODULE_PATHNAME';
 
+CREATE FUNCTION ag_catalog.age_create_barbell_graph(graph_name name, 
+                                                graph_size int, 
+                                                bridge_size int,
+                                                node_label name = NULL,
+                                                node_properties agtype = NULL,
+                                                edge_label name = NULL,
+                                                edge_properties agtype = NULL)
+RETURNS void
+LANGUAGE c
+CALLED ON NULL INPUT
+PARALLEL SAFE
+AS 'MODULE_PATHNAME';
+
 CREATE FUNCTION ag_catalog.age_prepare_cypher(cstring, cstring)
 RETURNS boolean
 LANGUAGE c
