@@ -19,9 +19,9 @@
 
 #include "postgres.h"
 
-#include "access/xact.h"
 #include "access/genam.h"
 #include "access/heapam.h"
+#include "access/xact.h"
 #include "catalog/dependency.h"
 #include "catalog/objectaddress.h"
 #include "commands/defrem.h"
@@ -35,9 +35,8 @@
 #include "nodes/pg_list.h"
 #include "nodes/value.h"
 #include "parser/parser.h"
-#include "utils/fmgroids.h"
-#include "utils/relcache.h"
 #include "utils/rel.h"
+#include "utils/relcache.h"
 
 #include "catalog/ag_graph.h"
 #include "catalog/ag_label.h"
@@ -58,6 +57,7 @@ static void rename_graph(const Name graph_name, const Name new_name);
 
 PG_FUNCTION_INFO_V1(create_graph);
 
+/* function that is evoked for creating a graph */
 Datum create_graph(PG_FUNCTION_ARGS)
 {
     char *graph;
@@ -65,20 +65,26 @@ Datum create_graph(PG_FUNCTION_ARGS)
     char *graph_name_str;
     Oid nsp_id;
 
+    //if no argument is passed with the function, graph name cannot be null
     if (PG_ARGISNULL(0))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                         errmsg("graph name can not be NULL")));
     }
-    graph_name = PG_GETARG_NAME(0);
+
+    //gets graph name as function argument
+    graph_name = PG_GETARG_NAME(0);  
 
     graph_name_str = NameStr(*graph_name);
 
+    //checking if the name of the graph falls under the pre-decided graph naming conventions(regex)
     if (!is_valid_graph_name(graph_name_str))
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                         errmsg("graph name is invalid")));
     }
+
+    //graph name must be unique, a graph with the same name should not exist
     if (graph_exists(graph_name_str))
     {
         ereport(ERROR,
@@ -88,7 +94,8 @@ Datum create_graph(PG_FUNCTION_ARGS)
 
     nsp_id = create_schema_for_graph(graph_name);
 
-    insert_graph(graph_name, nsp_id);
+    //inserts the graph info into the relation which has all the other existing graphs info
+    insert_graph(graph_name, nsp_id);  
 
     //Increment the Command counter before create the generic labels.
     CommandCounterIncrement();
@@ -101,7 +108,8 @@ Datum create_graph(PG_FUNCTION_ARGS)
     ereport(NOTICE,
             (errmsg("graph \"%s\" has been created", NameStr(*graph_name))));
 
-    PG_RETURN_VOID();
+    //according to postgres specification of c-language functions if function returns void this is the syntax
+    PG_RETURN_VOID(); 
 }
 
 static Oid create_schema_for_graph(const Name graph_name)
@@ -173,9 +181,8 @@ Datum drop_graph(PG_FUNCTION_ARGS)
     graph_name_str = NameStr(*graph_name);
     if (!graph_exists(graph_name_str))
     {
-        ereport(ERROR,
-                (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                 errmsg("graph \"%s\" does not exist", graph_name_str)));
+        ereport(ERROR, (errcode(ERRCODE_UNDEFINED_SCHEMA),
+                        errmsg("graph \"%s\" does not exist", graph_name_str)));
     }
 
     drop_schema_for_graph(graph_name_str, cascade);
@@ -358,11 +365,11 @@ List *get_graphnames(void)
     List *graphnames = NIL;
     char *str;
 
-    ag_graph = heap_open(ag_graph_relation_id(), RowExclusiveLock);
+    ag_graph = table_open(ag_graph_relation_id(), RowExclusiveLock);
     scan_desc = systable_beginscan(ag_graph, ag_graph_name_index_id(), true,
                                    NULL, 0, NULL);
 
-    slot = MakeTupleTableSlot(RelationGetDescr(ag_graph));
+    slot = MakeTupleTableSlot(RelationGetDescr(ag_graph), &TTSOpsHeapTuple);
 
     for (;;)
     {
@@ -371,17 +378,17 @@ List *get_graphnames(void)
             break;
 
         ExecClearTuple(slot);
-        ExecStoreTuple(tuple, slot, InvalidBuffer, false);
+        ExecStoreHeapTuple(tuple, slot, false);
 
         slot_getallattrs(slot);
 
-        str = DatumGetCString(slot->tts_values[0]);
+        str = DatumGetCString(slot->tts_values[Anum_ag_graph_name - 1]);
         graphnames = lappend(graphnames, str);
     }
 
     ExecDropSingleTupleTableSlot(slot);
     systable_endscan(scan_desc);
-    heap_close(ag_graph, RowExclusiveLock);
+    table_close(ag_graph, RowExclusiveLock);
 
     return graphnames;
 }
