@@ -49,6 +49,7 @@
 #include "commands/label_commands.h"
 #include "nodes/cypher_nodes.h"
 #include "parser/cypher_expr.h"
+#include "parser/cypher_item.h"
 #include "parser/cypher_parse_node.h"
 #include "parser/cypher_transform_entity.h"
 #include "utils/ag_func.h"
@@ -92,8 +93,8 @@ static Node *transform_CoalesceExpr(cypher_parsestate *cpstate,
                                     CoalesceExpr *cexpr);
 static Node *transform_SubLink(cypher_parsestate *cpstate, SubLink *sublink);
 static Node *transform_FuncCall(cypher_parsestate *cpstate, FuncCall *fn);
-static Node *transform_WholeRowRef(ParseState *pstate, RangeTblEntry *rte,
-                                   int location);
+static Node *transform_WholeRowRef(ParseState *pstate, ParseNamespaceItem *pnsi,
+                                   int location, int sublevels_up);
 static ArrayExpr *make_agtype_array_expr(List *args);
 
 /* transform a cypher expression */
@@ -147,6 +148,7 @@ static Node *transform_cypher_expr_recurse(cypher_parsestate *cpstate,
             ereport(ERROR, (errmsg_internal("unrecognized A_Expr kind: %d",
                                             a->kind)));
         }
+        break;
     }
     case T_BoolExpr:
         return transform_BoolExpr(cpstate, (BoolExpr *)expr);
@@ -256,15 +258,18 @@ static Node *transform_A_Const(cypher_parsestate *cpstate, A_Const *ac)
  * Private function borrowed from PG's transformWholeRowRef.
  * Construct a whole-row reference to represent the notation "relation.*".
  */
-static Node *transform_WholeRowRef(ParseState *pstate, RangeTblEntry *rte,
-                                   int location)
+static Node *transform_WholeRowRef(ParseState *pstate, ParseNamespaceItem *pnsi,
+                                   int location, int sublevels_up)
 {
     Var *result;
     int vnum;
-    int sublevels_up;
+    RangeTblEntry *rte;
+
+    Assert(pnsi->p_rte != NULL);
+    rte = pnsi->p_rte;
 
     /* Find the RTE's rangetable location */
-    vnum = RTERangeTablePosn(pstate, rte, &sublevels_up);
+    vnum = pnsi->p_rtindex;
 
     /*
      * Build the appropriate referencing node.  Note that if the RTE is a
@@ -274,7 +279,7 @@ static Node *transform_WholeRowRef(ParseState *pstate, RangeTblEntry *rte,
      * historically.  One argument for it is that "rel" and "rel.*" mean the
      * same thing for composite relations, so why not for scalar functions...
      */
-     result = makeWholeRowVar(rte, vnum, sublevels_up, true);
+     result = makeWholeRowVar(rte, vnum, sublevels_up, true); 
 
      /* location is not filled in by makeWholeRowVar */
      result->location = location;
@@ -292,7 +297,6 @@ static Node *transform_WholeRowRef(ParseState *pstate, RangeTblEntry *rte,
 static Node *transform_ColumnRef(cypher_parsestate *cpstate, ColumnRef *cref)
 {
     ParseState *pstate = (ParseState *)cpstate;
-    RangeTblEntry *rte = NULL;
     Node *field1 = NULL;
     Node *field2 = NULL;
     char *colname = NULL;
@@ -300,6 +304,7 @@ static Node *transform_ColumnRef(cypher_parsestate *cpstate, ColumnRef *cref)
     char *relname = NULL;
     Node *node = NULL;
     int levels_up;
+    ParseNamespaceItem *pnsi;
 
     switch (list_length(cref->fields))
     {
@@ -319,16 +324,18 @@ static Node *transform_ColumnRef(cypher_parsestate *cpstate, ColumnRef *cref)
                 }
 
                 /*
-                 * Try to find the columnRef as a transform_entity and extract
-                 * the expr.
+                 * If expr_kind is WHERE, Try to find the columnRef as a
+                 * transform_entity and extract the expr.
                  */
-                te = find_variable(cpstate, colname) ;
-                if (te != NULL && te->expr != NULL)
+                if (pstate->p_expr_kind == EXPR_KIND_WHERE)
                 {
-                    node = (Node *)te->expr;
-                    break;
+                    te = find_variable(cpstate, colname) ;
+                    if (te != NULL && te->expr != NULL)
+                    {
+                        node = (Node *)te->expr;
+                        break;
+                    }
                 }
-
                 /*
                  * Not known as a column of any range-table entry.
                  * Try to find the name as a relation.  Note that only
@@ -339,11 +346,12 @@ static Node *transform_ColumnRef(cypher_parsestate *cpstate, ColumnRef *cref)
                  * PostQUEL-inspired syntax.  The preferred form now is
                  * "rel.*".
                  */
-                rte = refnameRangeTblEntry(pstate, NULL, colname,
+                pnsi = refnameNamespaceItem(pstate, NULL, colname,
                                            cref->location, &levels_up);
-                if (rte)
+                if (pnsi)
                 {
-                    node = transform_WholeRowRef(pstate, rte, cref->location);
+                    node = transform_WholeRowRef(pstate, pnsi, cref->location,
+                                                 levels_up);
                 }
                 else
                 {
@@ -379,9 +387,10 @@ static Node *transform_ColumnRef(cypher_parsestate *cpstate, ColumnRef *cref)
                 }
 
                 /* locate the referenced RTE */
-                rte = refnameRangeTblEntry(pstate, nspname, relname,
+                pnsi = refnameNamespaceItem(pstate, nspname, relname,
                                            cref->location, &levels_up);
-                if (rte == NULL)
+
+                if (pnsi == NULL)
                 {
                     ereport(ERROR,
                             (errcode(ERRCODE_UNDEFINED_COLUMN),
@@ -397,15 +406,15 @@ static Node *transform_ColumnRef(cypher_parsestate *cpstate, ColumnRef *cref)
                  */
                 if (IsA(field2, A_Star))
                 {
-                    node = transform_WholeRowRef(pstate, rte, cref->location);
+                    node = transform_WholeRowRef(pstate, pnsi, cref->location, levels_up);
                     break;
                 }
 
                 Assert(IsA(field2, String));
 
                 /* try to identify as a column of the RTE */
-                node = scanRTEForColumn(pstate, rte, colname, cref->location, 0,
-                                        NULL);
+                node = scanNSItemForColumn(pstate, pnsi, 0, colname, cref->location);
+
                 if (node == NULL)
                 {
                     ereport(ERROR,
@@ -614,9 +623,9 @@ static Node *transform_cypher_map(cypher_parsestate *cpstate, cypher_map *cm)
         Const *newkey;
 
         key = lfirst(le);
-        le = lnext(le);
+        le = lnext(cm->keyvals, le);
         val = lfirst(le);
-        le = lnext(le);
+        le = lnext(cm->keyvals, le);
 
         newval = transform_cypher_expr_recurse(cpstate, val);
 
@@ -634,6 +643,10 @@ static Node *transform_cypher_map(cypher_parsestate *cpstate, cypher_map *cm)
     {
         func_oid = get_ag_func_oid("agtype_build_map", 0);
     }
+    else if (!cm->keep_null)
+    {
+        func_oid = get_ag_func_oid("agtype_build_map_nonull", 1, ANYOID);
+    }
     else
     {
         func_oid = get_ag_func_oid("agtype_build_map", 1, ANYOID);
