@@ -19,6 +19,8 @@
 
 #include "postgres.h"
 
+#include "access/heapam.h"
+#include "catalog/namespace.h"
 #include "catalog/pg_type.h"
 #include "funcapi.h"
 #include "utils/lsyscache.h"
