@@ -26,8 +26,8 @@
 #include "access/htup_details.h"
 #include "access/skey.h"
 #include "access/stratnum.h"
-#include "access/sysattr.h"
 #include "access/tupdesc.h"
+#include "catalog/pg_collation.h"
 #include "fmgr.h"
 #include "storage/lockdefs.h"
 #include "utils/builtins.h"
@@ -69,17 +69,17 @@ typedef struct label_name_graph_cache_entry
     label_cache_data data;
 } label_name_graph_cache_entry;
 
-typedef struct label_graph_id_cache_key
+typedef struct label_graph_oid_cache_key
 {
     Oid graph;
     int32 id;
-} label_graph_id_cache_key;
+} label_graph_oid_cache_key;
 
-typedef struct label_graph_id_cache_entry
+typedef struct label_graph_oid_cache_entry
 {
-    label_graph_id_cache_key key; // hash key
+    label_graph_oid_cache_key key; // hash key
     label_cache_data data;
-} label_graph_id_cache_entry;
+} label_graph_oid_cache_entry;
 
 typedef struct label_relation_cache_entry
 {
@@ -107,17 +107,13 @@ static ScanKeyData graph_name_scan_keys[1];
 static HTAB *graph_namespace_cache_hash = NULL;
 static ScanKeyData graph_namespace_scan_keys[1];
 
-// ag_label.oid
-static HTAB *label_oid_cache_hash = NULL;
-static ScanKeyData label_oid_scan_keys[1];
-
 // ag_label.name, ag_label.graph
 static HTAB *label_name_graph_cache_hash = NULL;
 static ScanKeyData label_name_graph_scan_keys[2];
 
 // ag_label.graph, ag_label.id
-static HTAB *label_graph_id_cache_hash = NULL;
-static ScanKeyData label_graph_id_scan_keys[2];
+static HTAB *label_graph_oid_cache_hash = NULL;
+static ScanKeyData label_graph_oid_scan_keys[2];
 
 // ag_label.relation
 static HTAB *label_relation_cache_hash = NULL;
@@ -152,38 +148,35 @@ static void fill_graph_cache_data(graph_cache_data *cache_data,
 // ag_label
 static void initialize_label_caches(void);
 static void create_label_caches(void);
-static void create_label_oid_cache(void);
 static void create_label_name_graph_cache(void);
-static void create_label_graph_id_cache(void);
+static void create_label_graph_oid_cache(void);
 static void create_label_relation_cache(void);
 static void create_label_seq_name_graph_cache(void);
 static void invalidate_label_caches(Datum arg, Oid relid);
-static void invalidate_label_oid_cache(Oid relid);
-static void flush_label_oid_cache(void);
 static void invalidate_label_name_graph_cache(Oid relid);
 static void flush_label_name_graph_cache(void);
-static void invalidate_label_graph_id_cache(Oid relid);
-static void flush_label_graph_id_cache(void);
+static void invalidate_label_graph_oid_cache(Oid relid);
+static void flush_label_graph_oid_cache(void);
 static void invalidate_label_relation_cache(Oid relid);
 static void flush_label_relation_cache(void);
 static void invalidate_label_seq_name_graph_cache(Oid relid);
 static void flush_label_seq_name_graph_cache(void);
 
-static label_cache_data *search_label_oid_cache_miss(Oid oid);
 static label_cache_data *search_label_name_graph_cache_miss(Name name,
                                                             Oid graph);
 static void *label_name_graph_cache_hash_search(Name name, Oid graph,
                                                 HASHACTION action,
                                                 bool *found);
-static label_cache_data *search_label_graph_id_cache_miss(Oid graph, int32 id);
-static void *label_graph_id_cache_hash_search(Oid graph, int32 id,
-                                              HASHACTION action, bool *found);
+static label_cache_data *search_label_graph_oid_cache_miss(Oid graph,
+                                                           uint32 id);
+static void *label_graph_oid_cache_hash_search(uint32 graph, int32 id,
+                                               HASHACTION action, bool *found);
 static label_cache_data *search_label_relation_cache_miss(Oid relation);
 static label_cache_data *search_label_seq_name_graph_cache_miss(Name name,
-                                                            Oid graph);
+                                                                Oid graph);
 static void *label_seq_name_graph_cache_hash_search(Name name, Oid graph,
-                                                HASHACTION action,
-                                                bool *found);
+                                                    HASHACTION action,
+                                                    bool *found);
 
 static void fill_label_cache_data(label_cache_data *cache_data,
                                   HeapTuple tuple, TupleDesc tuple_desc);
@@ -193,11 +186,13 @@ static void initialize_caches(void)
     static bool initialized = false;
 
     if (initialized)
+    {
         return;
-
+    }
     if (!CacheMemoryContext)
+    {
         CreateCacheMemoryContext();
-
+    }
     initialize_graph_caches();
     initialize_label_caches();
 
@@ -211,7 +206,7 @@ static void ag_cache_scan_key_init(ScanKey entry, AttrNumber attno,
     entry->sk_attno = attno;
     entry->sk_strategy = BTEqualStrategyNumber;
     entry->sk_subtype = InvalidOid;
-    entry->sk_collation = InvalidOid;
+    entry->sk_collation = C_COLLATION_OID;
     fmgr_info_cxt(func, &entry->sk_func, CacheMemoryContext);
     entry->sk_argument = (Datum)0;
 }
@@ -316,12 +311,15 @@ static void flush_graph_name_cache(void)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         removed = hash_search(graph_name_cache_hash, &entry->name, HASH_REMOVE,
                               NULL);
         if (!removed)
+        {
             ereport(ERROR, (errmsg_internal("graph (name) cache corrupted")));
+        }
     }
 }
 
@@ -337,7 +335,9 @@ static void flush_graph_namespace_cache(void)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
+        }
 
         removed = hash_search(graph_namespace_cache_hash, &entry->namespace,
                               HASH_REMOVE, NULL);
@@ -361,7 +361,9 @@ graph_cache_data *search_graph_name_cache(const char *name)
     namestrcpy(&name_key, name);
     entry = hash_search(graph_name_cache_hash, &name_key, HASH_FIND, NULL);
     if (entry)
+    {
         return &entry->data;
+    }
 
     return search_graph_name_cache_miss(&name_key);
 }
@@ -379,11 +381,11 @@ static graph_cache_data *search_graph_name_cache_miss(Name name)
     scan_keys[0].sk_argument = NameGetDatum(name);
 
     /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * Calling table_open() might call AcceptInvalidationMessage() and that
      * might flush the graph caches. This is OK because this function is called
      * when the desired entry is not in the cache.
      */
-    ag_graph = heap_open(ag_graph_relation_id(), AccessShareLock);
+    ag_graph = table_open(ag_graph_relation_id(), AccessShareLock);
     scan_desc = systable_beginscan(ag_graph, ag_graph_name_index_id(), true,
                                    NULL, 1, scan_keys);
 
@@ -392,7 +394,7 @@ static graph_cache_data *search_graph_name_cache_miss(Name name)
     if (!HeapTupleIsValid(tuple))
     {
         systable_endscan(scan_desc);
-        heap_close(ag_graph, AccessShareLock);
+        table_close(ag_graph, AccessShareLock);
 
         return NULL;
     }
@@ -405,7 +407,7 @@ static graph_cache_data *search_graph_name_cache_miss(Name name)
     fill_graph_cache_data(&entry->data, tuple, RelationGetDescr(ag_graph));
 
     systable_endscan(scan_desc);
-    heap_close(ag_graph, AccessShareLock);
+    table_close(ag_graph, AccessShareLock);
 
     return &entry->data;
 }
@@ -419,7 +421,9 @@ graph_cache_data *search_graph_namespace_cache(Oid namespace)
     entry = hash_search(graph_namespace_cache_hash, &namespace, HASH_FIND,
                         NULL);
     if (entry)
+    {
         return &entry->data;
+    }
 
     return search_graph_namespace_cache_miss(namespace);
 }
@@ -438,11 +442,11 @@ static graph_cache_data *search_graph_namespace_cache_miss(Oid namespace)
     scan_keys[0].sk_argument = ObjectIdGetDatum(namespace);
 
     /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * Calling table_open() might call AcceptInvalidationMessage() and that
      * might flush the graph caches. This is OK because this function is called
      * when the desired entry is not in the cache.
      */
-    ag_graph = heap_open(ag_graph_relation_id(), AccessShareLock);
+    ag_graph = table_open(ag_graph_relation_id(), AccessShareLock);
     scan_desc = systable_beginscan(ag_graph, ag_graph_namespace_index_id(),
                                    true, NULL, 1, scan_keys);
 
@@ -452,7 +456,7 @@ static graph_cache_data *search_graph_namespace_cache_miss(Oid namespace)
     if (!HeapTupleIsValid(tuple))
     {
         systable_endscan(scan_desc);
-        heap_close(ag_graph, AccessShareLock);
+        table_close(ag_graph, AccessShareLock);
 
         return NULL;
     }
@@ -466,7 +470,7 @@ static graph_cache_data *search_graph_namespace_cache_miss(Oid namespace)
     fill_graph_cache_data(&entry->data, tuple, RelationGetDescr(ag_graph));
 
     systable_endscan(scan_desc);
-    heap_close(ag_graph, AccessShareLock);
+    table_close(ag_graph, AccessShareLock);
 
     return &entry->data;
 }
@@ -477,8 +481,8 @@ static void fill_graph_cache_data(graph_cache_data *cache_data,
     bool is_null;
     Datum value;
 
-    // ag_graph.oid
-    value = heap_getattr(tuple, ObjectIdAttributeNumber, tuple_desc, &is_null);
+    // ag_graph.id
+    value = heap_getattr(tuple, Anum_ag_graph_oid, tuple_desc, &is_null);
     Assert(!is_null);
     cache_data->oid = DatumGetObjectId(value);
     // ag_graph.name
@@ -493,31 +497,27 @@ static void fill_graph_cache_data(graph_cache_data *cache_data,
 
 static void initialize_label_caches(void)
 {
-    // ag_label.oid
-    ag_cache_scan_key_init(&label_oid_scan_keys[0], ObjectIdAttributeNumber,
-                           F_OIDEQ);
-
     // ag_label.name, ag_label.graph
     ag_cache_scan_key_init(&label_name_graph_scan_keys[0], Anum_ag_label_name,
                            F_NAMEEQ);
     ag_cache_scan_key_init(&label_name_graph_scan_keys[1], Anum_ag_label_graph,
-                           F_OIDEQ);
+                           F_INT4EQ);
 
     // ag_label.graph, ag_label.id
-    ag_cache_scan_key_init(&label_graph_id_scan_keys[0], Anum_ag_label_graph,
-                           F_OIDEQ);
-    ag_cache_scan_key_init(&label_graph_id_scan_keys[1], Anum_ag_label_id,
+    ag_cache_scan_key_init(&label_graph_oid_scan_keys[0], Anum_ag_label_graph,
+                           F_INT4EQ);
+    ag_cache_scan_key_init(&label_graph_oid_scan_keys[1], Anum_ag_label_id,
                            F_INT4EQ);
 
     // ag_label.relation
     ag_cache_scan_key_init(&label_relation_scan_keys[0],
                            Anum_ag_label_relation, F_OIDEQ);
-    
+
     // ag_label.seq_name, ag_label.graph
-    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[0], Anum_ag_label_seq_name,
-                           F_NAMEEQ);
-    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[1], Anum_ag_label_graph,
-                           F_OIDEQ);
+    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[0],
+                           Anum_ag_label_seq_name, F_NAMEEQ);
+    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[1],
+                           Anum_ag_label_graph, F_OIDEQ);
 
     create_label_caches();
 
@@ -534,34 +534,13 @@ static void create_label_caches(void)
      * All the hash tables are created using their dedicated memory contexts
      * which are under TopMemoryContext.
      */
-    create_label_oid_cache();
     create_label_name_graph_cache();
-    create_label_graph_id_cache();
+    create_label_graph_oid_cache();
     create_label_relation_cache();
     create_label_seq_name_graph_cache();
 
 }
 
-static void create_label_oid_cache(void)
-{
-    HASHCTL hash_ctl;
-
-    /*
-     * Use label_cache_data itself since it has oid field as its first field
-     * that is the key for this hash.
-     */
-    MemSet(&hash_ctl, 0, sizeof(hash_ctl));
-    hash_ctl.keysize = sizeof(Oid);
-    hash_ctl.entrysize = sizeof(label_cache_data);
-
-    /*
-     * Please see the comment of hash_create() for the nelem value 16 here.
-     * HASH_BLOBS flag is set because the size of the key is sizeof(uint32).
-     */
-    label_oid_cache_hash = hash_create("ag_label (oid) cache", 16, &hash_ctl,
-                                       HASH_ELEM | HASH_BLOBS);
-}
-
 static void create_label_name_graph_cache(void)
 {
     HASHCTL hash_ctl;
@@ -579,21 +558,21 @@ static void create_label_name_graph_cache(void)
                                               HASH_ELEM | HASH_BLOBS);
 }
 
-static void create_label_graph_id_cache(void)
+static void create_label_graph_oid_cache(void)
 {
     HASHCTL hash_ctl;
 
     MemSet(&hash_ctl, 0, sizeof(hash_ctl));
-    hash_ctl.keysize = sizeof(label_graph_id_cache_key);
-    hash_ctl.entrysize = sizeof(label_graph_id_cache_entry);
+    hash_ctl.keysize = sizeof(label_graph_oid_cache_key);
+    hash_ctl.entrysize = sizeof(label_graph_oid_cache_entry);
 
     /*
      * Please see the comment of hash_create() for the nelem value 16 here.
      * HASH_BLOBS flag is set because the key for this hash is fixed-size.
      */
-    label_graph_id_cache_hash = hash_create("ag_label (graph, id) cache", 16,
-                                              &hash_ctl,
-                                              HASH_ELEM | HASH_BLOBS);
+    label_graph_oid_cache_hash = hash_create("ag_label (graph, id) cache", 16,
+                                             &hash_ctl,
+                                             HASH_ELEM | HASH_BLOBS);
 }
 
 static void create_label_relation_cache(void)
@@ -625,8 +604,8 @@ static void create_label_seq_name_graph_cache(void)
      * HASH_BLOBS flag is set because the key for this hash is fixed-size.
      */
     label_seq_name_graph_cache_hash = hash_create("ag_label (seq_name, graph) cache",
-                                              16, &hash_ctl,
-                                              HASH_ELEM | HASH_BLOBS);
+                                                  16, &hash_ctl,
+                                                  HASH_ELEM | HASH_BLOBS);
 }
 
 static void invalidate_label_caches(Datum arg, Oid relid)
@@ -637,77 +616,20 @@ static void invalidate_label_caches(Datum arg, Oid relid)
 
     if (OidIsValid(relid))
     {
-        invalidate_label_oid_cache(relid);
         invalidate_label_name_graph_cache(relid);
-        invalidate_label_graph_id_cache(relid);
+        invalidate_label_graph_oid_cache(relid);
         invalidate_label_relation_cache(relid);
         invalidate_label_seq_name_graph_cache(relid);
     }
     else
     {
-        flush_label_oid_cache();
         flush_label_name_graph_cache();
-        flush_label_graph_id_cache();
+        flush_label_graph_oid_cache();
         flush_label_relation_cache();
         flush_label_seq_name_graph_cache();
     }
 }
 
-static void invalidate_label_oid_cache(Oid relid)
-{
-    HASH_SEQ_STATUS hash_seq;
-
-    hash_seq_init(&hash_seq, label_oid_cache_hash);
-    for (;;)
-    {
-        label_cache_data *entry;
-        void *removed;
-
-        entry = hash_seq_search(&hash_seq);
-        if (!entry)
-            break;
-
-        if (entry->relation != relid)
-            continue;
-
-        removed = hash_search(label_oid_cache_hash, &entry->oid, HASH_REMOVE,
-                              NULL);
-        hash_seq_term(&hash_seq);
-
-        if (!removed)
-        {
-            ereport(ERROR,
-                    (errmsg_internal("label (oid) cache corrupted")));
-        }
-
-        break;
-    }
-}
-
-static void flush_label_oid_cache(void)
-{
-    HASH_SEQ_STATUS hash_seq;
-
-    hash_seq_init(&hash_seq, label_oid_cache_hash);
-    for (;;)
-    {
-        label_cache_data *entry;
-        void *removed;
-
-        entry = hash_seq_search(&hash_seq);
-        if (!entry)
-            break;
-
-        removed = hash_search(label_oid_cache_hash, &entry->oid, HASH_REMOVE,
-                              NULL);
-        if (!removed)
-        {
-            ereport(ERROR,
-                    (errmsg_internal("label (oid) cache corrupted")));
-        }
-    }
-}
-
 static void invalidate_label_name_graph_cache(Oid relid)
 {
     HASH_SEQ_STATUS hash_seq;
@@ -720,11 +642,13 @@ static void invalidate_label_name_graph_cache(Oid relid)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         if (entry->data.relation != relid)
+        {
             continue;
-
+        }
         removed = hash_search(label_name_graph_cache_hash, &entry->key,
                               HASH_REMOVE, NULL);
         hash_seq_term(&hash_seq);
@@ -751,8 +675,9 @@ static void flush_label_name_graph_cache(void)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         removed = hash_search(label_name_graph_cache_hash, &entry->key,
                               HASH_REMOVE, NULL);
         if (!removed)
@@ -763,24 +688,26 @@ static void flush_label_name_graph_cache(void)
     }
 }
 
-static void invalidate_label_graph_id_cache(Oid relid)
+static void invalidate_label_graph_oid_cache(Oid relid)
 {
     HASH_SEQ_STATUS hash_seq;
 
-    hash_seq_init(&hash_seq, label_graph_id_cache_hash);
+    hash_seq_init(&hash_seq, label_graph_oid_cache_hash);
     for (;;)
     {
-        label_graph_id_cache_entry *entry;
+        label_graph_oid_cache_entry *entry;
         void *removed;
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         if (entry->data.relation != relid)
+        {
             continue;
-
-        removed = hash_search(label_graph_id_cache_hash, &entry->key,
+        }
+        removed = hash_search(label_graph_oid_cache_hash, &entry->key,
                               HASH_REMOVE, NULL);
         hash_seq_term(&hash_seq);
 
@@ -794,21 +721,22 @@ static void invalidate_label_graph_id_cache(Oid relid)
     }
 }
 
-static void flush_label_graph_id_cache(void)
+static void flush_label_graph_oid_cache(void)
 {
     HASH_SEQ_STATUS hash_seq;
 
-    hash_seq_init(&hash_seq, label_graph_id_cache_hash);
+    hash_seq_init(&hash_seq, label_graph_oid_cache_hash);
     for (;;)
     {
-        label_graph_id_cache_entry *entry;
+        label_graph_oid_cache_entry *entry;
         void *removed;
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
-        removed = hash_search(label_graph_id_cache_hash, &entry->key,
+        }
+        removed = hash_search(label_graph_oid_cache_hash, &entry->key,
                               HASH_REMOVE, NULL);
         if (!removed)
         {
@@ -825,12 +753,15 @@ static void invalidate_label_relation_cache(Oid relid)
 
     entry = hash_search(label_relation_cache_hash, &relid, HASH_FIND, NULL);
     if (!entry)
+    {
         return;
-
+    }
     removed = hash_search(label_relation_cache_hash, &relid, HASH_REMOVE,
                           NULL);
     if (!removed)
+    {
         ereport(ERROR, (errmsg_internal("label (namespace) cache corrupted")));
+    }
 }
 
 static void flush_label_relation_cache(void)
@@ -845,8 +776,9 @@ static void flush_label_relation_cache(void)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         removed = hash_search(label_relation_cache_hash, &entry->relation,
                               HASH_REMOVE, NULL);
         if (!removed)
@@ -869,11 +801,13 @@ static void invalidate_label_seq_name_graph_cache(Oid relid)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         if (entry->data.relation != relid)
+        {
             continue;
-
+        }
         removed = hash_search(label_seq_name_graph_cache_hash, &entry->key,
                               HASH_REMOVE, NULL);
         hash_seq_term(&hash_seq);
@@ -900,8 +834,9 @@ static void flush_label_seq_name_graph_cache(void)
 
         entry = hash_seq_search(&hash_seq);
         if (!entry)
+        {
             break;
-
+        }
         removed = hash_search(label_seq_name_graph_cache_hash, &entry->key,
                               HASH_REMOVE, NULL);
         if (!removed)
@@ -912,72 +847,12 @@ static void flush_label_seq_name_graph_cache(void)
     }
 }
 
-label_cache_data *search_label_oid_cache(Oid oid)
-{
-    label_cache_data *entry;
-
-    initialize_caches();
-
-    entry = hash_search(label_oid_cache_hash, &oid, HASH_FIND, NULL);
-    if (entry)
-        return entry;
-
-    return search_label_oid_cache_miss(oid);
-}
-
-static label_cache_data *search_label_oid_cache_miss(Oid oid)
-{
-    ScanKeyData scan_keys[1];
-    Relation ag_label;
-    SysScanDesc scan_desc;
-    HeapTuple tuple;
-    bool found;
-    label_cache_data *entry;
-
-    memcpy(scan_keys, label_oid_scan_keys, sizeof(label_oid_scan_keys));
-    scan_keys[0].sk_argument = ObjectIdGetDatum(oid);
-
-    /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
-     * might invalidate the label caches. This is OK because this function is
-     * called when the desired entry is not in the cache.
-     */
-    ag_label = heap_open(ag_label_relation_id(), AccessShareLock);
-    scan_desc = systable_beginscan(ag_label, ag_label_oid_index_id(), true,
-                                   NULL, 1, scan_keys);
-
-    // don't need to loop over scan_desc because ag_label_oid_index is UNIQUE
-    tuple = systable_getnext(scan_desc);
-    if (!HeapTupleIsValid(tuple))
-    {
-        systable_endscan(scan_desc);
-        heap_close(ag_label, AccessShareLock);
-
-        return NULL;
-    }
-
-    // get a new entry
-    entry = hash_search(label_oid_cache_hash, &oid, HASH_ENTER, &found);
-    Assert(!found); // no concurrent update on label_oid_cache_hash
-
-    // fill the new entry with the retrieved tuple
-    fill_label_cache_data(entry, tuple, RelationGetDescr(ag_label));
-    // make sure that the oid field is the same with the hash key(oid)
-    Assert(entry->oid == oid);
-
-    systable_endscan(scan_desc);
-    heap_close(ag_label, AccessShareLock);
-
-    return entry;
-}
-
 label_cache_data *search_label_name_graph_cache(const char *name, Oid graph)
 {
     NameData name_key;
     label_name_graph_cache_entry *entry;
 
     AssertArg(name);
-    AssertArg(OidIsValid(graph));
 
     initialize_caches();
 
@@ -985,8 +860,9 @@ label_cache_data *search_label_name_graph_cache(const char *name, Oid graph)
     entry = label_name_graph_cache_hash_search(&name_key, graph, HASH_FIND,
                                                NULL);
     if (entry)
+    {
         return &entry->data;
-
+    }
     return search_label_name_graph_cache_miss(&name_key, graph);
 }
 
@@ -1006,11 +882,11 @@ static label_cache_data *search_label_name_graph_cache_miss(Name name,
     scan_keys[1].sk_argument = ObjectIdGetDatum(graph);
 
     /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * Calling table_open() might call AcceptInvalidationMessage() and that
      * might invalidate the label caches. This is OK because this function is
      * called when the desired entry is not in the cache.
      */
-    ag_label = heap_open(ag_label_relation_id(), AccessShareLock);
+    ag_label = table_open(ag_label_relation_id(), AccessShareLock);
     scan_desc = systable_beginscan(ag_label, ag_label_name_graph_index_id(),
                                    true, NULL, 2, scan_keys);
 
@@ -1022,7 +898,7 @@ static label_cache_data *search_label_name_graph_cache_miss(Name name,
     if (!HeapTupleIsValid(tuple))
     {
         systable_endscan(scan_desc);
-        heap_close(ag_label, AccessShareLock);
+        table_close(ag_label, AccessShareLock);
 
         return NULL;
     }
@@ -1036,7 +912,7 @@ static label_cache_data *search_label_name_graph_cache_miss(Name name,
     fill_label_cache_data(&entry->data, tuple, RelationGetDescr(ag_label));
 
     systable_endscan(scan_desc);
-    heap_close(ag_label, AccessShareLock);
+    table_close(ag_label, AccessShareLock);
 
     return &entry->data;
 }
@@ -1053,81 +929,81 @@ static void *label_name_graph_cache_hash_search(Name name, Oid graph,
     return hash_search(label_name_graph_cache_hash, &key, action, found);
 }
 
-label_cache_data *search_label_graph_id_cache(Oid graph, int32 id)
+label_cache_data *search_label_graph_oid_cache(uint32 graph_oid, int32 id)
 {
-    label_graph_id_cache_entry *entry;
+    label_graph_oid_cache_entry *entry;
 
-    AssertArg(OidIsValid(graph));
     AssertArg(label_id_is_valid(id));
 
     initialize_caches();
 
-    entry = label_graph_id_cache_hash_search(graph, id, HASH_FIND, NULL);
+    entry = label_graph_oid_cache_hash_search(graph_oid, id, HASH_FIND, NULL);
     if (entry)
+    {
         return &entry->data;
-
-    return search_label_graph_id_cache_miss(graph, id);
+    }
+    return search_label_graph_oid_cache_miss(graph_oid, id);
 }
 
-static label_cache_data *search_label_graph_id_cache_miss(Oid graph, int32 id)
+static label_cache_data *search_label_graph_oid_cache_miss(Oid graph, uint32 id)
 {
     ScanKeyData scan_keys[2];
     Relation ag_label;
     SysScanDesc scan_desc;
     HeapTuple tuple;
     bool found;
-    label_graph_id_cache_entry *entry;
+    label_graph_oid_cache_entry *entry;
 
-    memcpy(scan_keys, label_graph_id_scan_keys,
-           sizeof(label_graph_id_scan_keys));
+    memcpy(scan_keys, label_graph_oid_scan_keys,
+           sizeof(label_graph_oid_scan_keys));
     scan_keys[0].sk_argument = ObjectIdGetDatum(graph);
     scan_keys[1].sk_argument = Int32GetDatum(id);
 
     /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * Calling table_open() might call AcceptInvalidationMessage() and that
      * might invalidate the label caches. This is OK because this function is
      * called when the desired entry is not in the cache.
      */
-    ag_label = heap_open(ag_label_relation_id(), AccessShareLock);
-    scan_desc = systable_beginscan(ag_label, ag_label_graph_id_index_id(),
-                                   true, NULL, 2, scan_keys);
+    ag_label = table_open(ag_label_relation_id(), AccessShareLock);
+    scan_desc = systable_beginscan(ag_label, ag_label_graph_oid_index_id(), true,
+                                   NULL, 2, scan_keys);
 
     /*
-     * don't need to loop over scan_desc because ag_label_graph_id_index is
+     * don't need to loop over scan_desc because ag_label_graph_oid_index is
      * UNIQUE
      */
     tuple = systable_getnext(scan_desc);
     if (!HeapTupleIsValid(tuple))
     {
         systable_endscan(scan_desc);
-        heap_close(ag_label, AccessShareLock);
+        table_close(ag_label, AccessShareLock);
 
         return NULL;
     }
 
     // get a new entry
-    entry = label_graph_id_cache_hash_search(graph, id, HASH_ENTER, &found);
-    Assert(!found); // no concurrent update on label_graph_id_cache_hash
+    entry = label_graph_oid_cache_hash_search(graph, id, HASH_ENTER, &found);
+    Assert(!found); // no concurrent update on label_graph_oid_cache_hash
 
     // fill the new entry with the retrieved tuple
     fill_label_cache_data(&entry->data, tuple, RelationGetDescr(ag_label));
 
     systable_endscan(scan_desc);
-    heap_close(ag_label, AccessShareLock);
+    table_close(ag_label, AccessShareLock);
 
     return &entry->data;
 }
 
-static void *label_graph_id_cache_hash_search(Oid graph, int32 id,
-                                              HASHACTION action, bool *found)
+static void *label_graph_oid_cache_hash_search(uint32 graph, int32 id,
+                                               HASHACTION action, bool *found)
 {
-    label_graph_id_cache_key key;
+    label_graph_oid_cache_key key;
 
-    // initialize the hash key for label_graph_id_cache_hash
+    // initialize the hash key for label_graph_oid_cache_hash
     key.graph = graph;
     key.id = id;
 
-    return hash_search(label_graph_id_cache_hash, &key, action, found);
+    return hash_search(label_graph_oid_cache_hash, &key, action, found);
 }
 
 label_cache_data *search_label_relation_cache(Oid relation)
@@ -1138,8 +1014,9 @@ label_cache_data *search_label_relation_cache(Oid relation)
 
     entry = hash_search(label_relation_cache_hash, &relation, HASH_FIND, NULL);
     if (entry)
+    {
         return &entry->data;
-
+    }
     return search_label_relation_cache_miss(relation);
 }
 
@@ -1157,13 +1034,13 @@ static label_cache_data *search_label_relation_cache_miss(Oid relation)
     scan_keys[0].sk_argument = ObjectIdGetDatum(relation);
 
     /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * Calling table_open() might call AcceptInvalidationMessage() and that
      * might invalidate the label caches. This is OK because this function is
      * called when the desired entry is not in the cache.
      */
-    ag_label = heap_open(ag_label_relation_id(), AccessShareLock);
-    scan_desc = systable_beginscan(ag_label, ag_label_relation_index_id(),
-                                   true, NULL, 1, scan_keys);
+    ag_label = table_open(ag_label_relation_id(), AccessShareLock);
+    scan_desc = systable_beginscan(ag_label, ag_label_relation_index_id(), true,
+                                   NULL, 1, scan_keys);
 
     // don't need to loop over scan_desc because ag_label_relation_index is
     // UNIQUE
@@ -1171,7 +1048,7 @@ static label_cache_data *search_label_relation_cache_miss(Oid relation)
     if (!HeapTupleIsValid(tuple))
     {
         systable_endscan(scan_desc);
-        heap_close(ag_label, AccessShareLock);
+        table_close(ag_label, AccessShareLock);
 
         return NULL;
     }
@@ -1185,7 +1062,7 @@ static label_cache_data *search_label_relation_cache_miss(Oid relation)
     fill_label_cache_data(entry, tuple, RelationGetDescr(ag_label));
 
     systable_endscan(scan_desc);
-    heap_close(ag_label, AccessShareLock);
+    table_close(ag_label, AccessShareLock);
 
     return entry;
 }
@@ -1202,15 +1079,16 @@ label_cache_data *search_label_seq_name_graph_cache(const char *name, Oid graph)
 
     namestrcpy(&name_key, name);
     entry = label_seq_name_graph_cache_hash_search(&name_key, graph, HASH_FIND,
-                                               NULL);
+                                                   NULL);
     if (entry)
+    {
         return &entry->data;
-
+    }
     return search_label_seq_name_graph_cache_miss(&name_key, graph);
 }
 
 static label_cache_data *search_label_seq_name_graph_cache_miss(Name name,
-                                                            Oid graph)
+                                                                Oid graph)
 {
     ScanKeyData scan_keys[2];
     Relation ag_label;
@@ -1225,11 +1103,11 @@ static label_cache_data *search_label_seq_name_graph_cache_miss(Name name,
     scan_keys[1].sk_argument = ObjectIdGetDatum(graph);
 
     /*
-     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * Calling table_open() might call AcceptInvalidationMessage() and that
      * might invalidate the label caches. This is OK because this function is
      * called when the desired entry is not in the cache.
      */
-    ag_label = heap_open(ag_label_relation_id(), AccessShareLock);
+    ag_label = table_open(ag_label_relation_id(), AccessShareLock);
     scan_desc = systable_beginscan(ag_label, ag_label_seq_name_graph_index_id(),
                                    true, NULL, 2, scan_keys);
 
@@ -1241,27 +1119,28 @@ static label_cache_data *search_label_seq_name_graph_cache_miss(Name name,
     if (!HeapTupleIsValid(tuple))
     {
         systable_endscan(scan_desc);
-        heap_close(ag_label, AccessShareLock);
+        table_close(ag_label, AccessShareLock);
 
         return NULL;
     }
 
     // get a new entry
     entry = label_seq_name_graph_cache_hash_search(name, graph, HASH_ENTER,
-                                               &found);
+                                                   &found);
     Assert(!found); // no concurrent update on label_seq_name_graph_cache_hash
 
     // fill the new entry with the retrieved tuple
     fill_label_cache_data(&entry->data, tuple, RelationGetDescr(ag_label));
 
     systable_endscan(scan_desc);
-    heap_close(ag_label, AccessShareLock);
+    table_close(ag_label, AccessShareLock);
 
     return &entry->data;
 }
 
 static void *label_seq_name_graph_cache_hash_search(Name name, Oid graph,
-                                                HASHACTION action, bool *found)
+                                                    HASHACTION action,
+                                                    bool *found)
 {
     label_seq_name_graph_cache_key key;
 
@@ -1278,10 +1157,6 @@ static void fill_label_cache_data(label_cache_data *cache_data,
     bool is_null;
     Datum value;
 
-    // ag_label.oid
-    value = heap_getattr(tuple, ObjectIdAttributeNumber, tuple_desc, &is_null);
-    Assert(!is_null);
-    cache_data->oid = DatumGetObjectId(value);
     // ag_label.name
     value = heap_getattr(tuple, Anum_ag_label_name, tuple_desc, &is_null);
     Assert(!is_null);
