@@ -31,8 +31,8 @@
 #ifndef AG_AGTYPE_H
 #define AG_AGTYPE_H
 
-#include "fmgr.h"
 #include "access/htup_details.h"
+#include "fmgr.h"
 #include "lib/stringinfo.h"
 #include "nodes/pg_list.h"
 #include "utils/array.h"
@@ -40,6 +40,7 @@
 #include "utils/syscache.h"
 
 #include "catalog/ag_namespace.h"
+#include "catalog/pg_type.h"
 #include "utils/graphid.h"
 
 /* Tokens used when sequentially processing an agtype value */
@@ -523,8 +524,9 @@ bool is_decimal_needed(char *numstr);
 int compare_agtype_scalar_values(agtype_value *a, agtype_value *b);
 agtype_value *alter_property_value(agtype_value *properties, char *var_name,
                                    agtype *new_v, bool remove_property);
-agtype_value *get_map_from_agtype(agtype *a);
-
+void remove_null_from_agtype_object(agtype_value *object);
+agtype_value *alter_properties(agtype_value *original_properties,
+                               agtype *new_properties);
 agtype *get_one_agtype_from_variadic_args(FunctionCallInfo fcinfo,
                                           int variadic_offset,
                                           int expected_nargs);
